@@ -30,6 +30,7 @@
 #include <iostream>
 #include <string>
 #include <algorithm>
+#include <iomanip>
 
 #include "output.h"
 #include "libnormaliz/general.h"
@@ -57,6 +58,8 @@ Output<Integer>::Output(){
     mod=false;
     msp=false;
     lattice_ideal_input = false;
+    no_ext_rays_output=false;
+    no_supp_hyps_output=false;
 }
 
 //---------------------------------------------------------------------------
@@ -68,6 +71,20 @@ void Output<Integer>::set_lattice_ideal_input(bool value){
 
 //---------------------------------------------------------------------------
 
+template<typename Integer>
+void Output<Integer>::set_no_supp_hyps_output(){
+    no_supp_hyps_output=true;
+}
+
+//---------------------------------------------------------------------------
+
+template<typename Integer>
+void Output<Integer>::set_no_ext_rays_output(){
+    no_ext_rays_output=true;
+}
+
+//---------------------------------------------------------------------------
+
 template<typename Integer>
 void Output<Integer>::read() const{
     cout<<"\nname="<<name<<"\n";
@@ -104,10 +121,15 @@ void Output<Integer>::setCone(Cone<Integer> & C) {
         of_cone       = "";
         of_monoid     = "";
         of_polyhedron = "";
+        module_generators_name = " lattice points in polytope (Hilbert basis elements of degree 1)";
     } else {
         of_cone       = " of recession cone";
         of_monoid     = " of recession monoid";
         of_polyhedron = " of polyhedron (homogenized)";
+        if(Result->isComputed(ConeProperty::ModuleGenerators) && Result->getRecessionRank()==0)
+            module_generators_name=" lattice points in polytope (module generators)";
+        else
+            module_generators_name=" module generators"; 
     }
 }
 
@@ -374,15 +396,15 @@ void Output<Integer>::write_Stanley_dec() const {
             for (size_t i=0; i<InExData.size(); ++i) {
                 out << InExData[i].first.size() << " ";
                 for (size_t j=0; j<InExData[i].first.size(); ++j) {
-                    out << InExData[i].first[j] << " ";
+                    out << InExData[i].first[j]+1 << " ";
                 }
                 out << InExData[i].second << endl;  
             }
         }
 
         out << "Stanley_dec" << endl;
-        const list<STANLEYDATA<Integer> >& StanleyDec = Result->getStanleyDec();
-        typename list<STANLEYDATA<Integer> >::const_iterator S = StanleyDec.begin();
+        list<STANLEYDATA_int>& StanleyDec = Result->getStanleyDec_mutable();
+        auto S = StanleyDec.begin();
         size_t i;
 
         out << StanleyDec.size() << endl; 
@@ -511,6 +533,48 @@ void Output<Integer>::write_inv_file() const{
                 inv <<"integer multiplicity = "      << mult.get_num() << endl;
                 inv <<"integer multiplicity_denom = "<< mult.get_den() << endl;
             }
+            if (Result->isComputed(ConeProperty::Volume)){
+                mpq_class vol = Result->getVolume();
+                inv <<"integer volume = "      << vol.get_num() << endl;
+                inv <<"integer volume_denom = "<< vol.get_den() << endl;
+            }
+            if (Result->isComputed(ConeProperty::WeightedEhrhartSeries)){
+                const HilbertSeries& HS=Result->getIntData().getWeightedEhrhartSeries().first;
+                
+                inv <<"vector "<< HS.getNum().size() <<" weighted_ehrhart_series_num = " << HS.getNum();
+                inv <<"integer num_common_denom = " << Result->getIntData().getWeightedEhrhartSeries().second << endl;
+                inv <<"vector "<< to_vector(HS.getDenom()).size() <<" weighted_ehrhart_series_num = " 
+                        << to_vector(HS.getDenom());
+                Result->getIntData().computeWeightedEhrhartQuasiPolynomial();
+                if (Result->getIntData().isWeightedEhrhartQuasiPolynomialComputed()) {
+                    if(HS.get_nr_coeff_quasipol()>=0){
+                        inv << "integer nr_coeff_weightedEhrhart__quasipol = " << HS.get_nr_coeff_quasipol() << endl;                       
+                    }
+                    vector< vector <mpz_class> > hqp= Result->getIntData().getWeightedEhrhartQuasiPolynomial();
+                    inv << "matrix " << hqp.size() << " " << hqp[0].size()
+                        << " weighted_ehrhart_quasipolynomial = ";
+                    inv << endl << hqp;
+                    inv << "integer weighted_ehrhart_quasipolynomial_denom = " 
+                        << Result->getIntData().getWeightedEhrhartQuasiPolynomialDenom() << endl;
+                }
+                if(HS.get_expansion_degree()>-1){
+                    vector<mpz_class> expansion=HS.getExpansion();
+                    inv << "vector weighted_ehrhart_series_expansion " << expansion.size() << " = " << expansion;
+                    inv <<"integer expansion_coeff_common_denom = " << Result->getIntData().getWeightedEhrhartSeries().second << endl;
+                }
+                
+            }
+            
+            if (Result->isComputed(ConeProperty::VirtualMultiplicity)){
+                mpq_class mult = Result->getVirtualMultiplicity();
+                inv <<"integer virtual_multiplicity = "      << mult.get_num() << endl;
+                inv <<"integer virtual_multiplicity_denom = "<< mult.get_den() << endl;
+            }
+            if (Result->isComputed(ConeProperty::Integral)){
+                mpq_class mult = Result->getIntegral();
+                inv <<"integer integral = "      << mult.get_num() << endl;
+                inv <<"integer integral_denom = "<< mult.get_den() << endl;
+            }
             if (Result->isComputed(ConeProperty::HilbertSeries)) {
                 const HilbertSeries& HS = Result->getHilbertSeries();
                 vector<mpz_class> HSnum;
@@ -529,6 +593,9 @@ void Output<Integer>::write_inv_file() const{
                     inv << HSdenom;
                 HS.computeHilbertQuasiPolynomial();
                 if (HS.isHilbertQuasiPolynomialComputed()) {
+                    if(HS.get_nr_coeff_quasipol()>=0){
+                        inv << "integer nr_coeff_hilbert_quasipol = " << HS.get_nr_coeff_quasipol() << endl;                       
+                    }
                     vector< vector <mpz_class> > hqp = HS.getHilbertQuasiPolynomial();
                     inv << "matrix " << hqp.size() << " " << hqp[0].size()
                         << " hilbert_quasipolynomial = ";
@@ -536,6 +603,10 @@ void Output<Integer>::write_inv_file() const{
                     inv << "integer hilbert_quasipolynomial_denom = " 
                         << HS.getHilbertQuasiPolynomialDenom() << endl;
                 }
+                if(HS.get_expansion_degree()>-1){
+                    vector<mpz_class> expansion=HS.getExpansion();
+                    inv << "vector Hilbert_series_expansion " << expansion.size() << " = " << expansion;
+                }
             }
         }
         if (Result->isComputed(ConeProperty::IsReesPrimary)) {
@@ -553,10 +624,110 @@ void Output<Integer>::write_inv_file() const{
             inv <<"vector "<< Result->getClassGroup().size() <<" class_group = "<< Result->getClassGroup();
         }
         
+        if (Result->isComputed(ConeProperty::IsGorenstein)) {
+            if(Result->isGorenstein()){
+                inv << "boolean Gorenstein = true" << endl;
+                inv <<"vector "<< Result->getGeneratorOfInterior().size() <<" generator_of_interior = "<< Result->getGeneratorOfInterior();                
+            }
+            else
+                inv << "boolean Gorenstein = false" << endl;
+        }
+        
         inv.close();
     }
 }
 
+//---------------------------------------------------------------------------
+
+template<typename Integer>
+void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const{
+
+    HilbertSeries HS=Result->getIntData().getWeightedEhrhartSeries().first;
+    out << "Weighted Ehrhart series:" << endl;
+    vector<mpz_class> num( HS.getNum());
+    for(size_t i=0;i<num.size();++i)
+        out << num[i] << " ";
+    out << endl << "Common denominator of coefficients: ";
+    out << Result->getIntData().getWeightedEhrhartSeries().second << endl;
+    map<long, long> HS_Denom = HS.getDenom();
+    long nr_factors = 0;
+    for (map<long, long>::iterator it = HS_Denom.begin(); it!=HS_Denom.end(); ++it) {
+        nr_factors += it->second;
+    }
+    out << "Series denominator with " << nr_factors << " factors:" << endl;
+    out << HS.getDenom();
+    if (HS.getShift() != 0) {
+        out << "shift = " << HS.getShift() << endl << endl;
+    }
+    out << "degree of weighted Ehrhart series as rational function = "
+            << HS.getDegreeAsRationalFunction() << endl << endl;
+            
+    if(HS.get_expansion_degree()>-1){
+        vector<mpz_class> expansion=HS.getExpansion();
+        out << "Expansion of weighted Ehrhart series" << endl;
+        for(size_t i=0;i<expansion.size();++i)
+            out << i+HS.getShift()  << ": " << expansion[i] << endl;
+        out << "Common denominator of coefficients: ";
+        out << Result->getIntData().getWeightedEhrhartSeries().second << endl;
+        out << endl;
+    }
+    
+    long period = HS.getPeriod();
+    if (period == 1) {
+        out << "Weighted Ehrhart polynomial:" << endl;
+        for(size_t i=0; i<HS.getHilbertQuasiPolynomial()[0].size();++i)
+            out << HS.getHilbertQuasiPolynomial()[0][i] << " ";
+        out << endl;
+        out << "with common denominator: ";
+        out << HS.getHilbertQuasiPolynomialDenom()*Result->getIntData().getNumeratorCommonDenom();
+    } else {
+        // output cyclonomic representation
+        out << "Weighted Ehrhart series with cyclotomic denominator:" << endl;
+        num=HS.getCyclotomicNum();
+        for(size_t i=0;i<num.size();++i)
+            out << num[i] << " ";
+        out << endl << "Common denominator of coefficients: ";
+        out << Result->getIntData().getWeightedEhrhartSeries().second << endl;
+        out << "Series cyclotomic denominator:" << endl;
+        out << HS.getCyclotomicDenom();
+        out << endl;
+        // Weighted Ehrhart quasi-polynomial
+        // vector< vector<mpz_class> > hilbert_quasi_poly = HS.getHilbertQuasiPolynomial();
+        if (HS.isHilbertQuasiPolynomialComputed()) { 
+            out<<"Weighted Ehrhart quasi-polynomial of period " << period << ":" << endl;
+            if(HS.get_nr_coeff_quasipol()>=0){
+                out << "only " << HS.get_nr_coeff_quasipol() << " highest coefficients computed" << endl;
+                out << "their common period is " << HS.getHilbertQuasiPolynomial().size() << "." << endl;                        
+            }
+            Matrix<mpz_class> HQP(HS.getHilbertQuasiPolynomial());
+            HQP.pretty_print(out,true);
+            out<<"with common denominator: "
+               <<Result->getIntData().getWeightedEhrhartQuasiPolynomialDenom() << endl;
+        }
+        else{
+            out<<"Weighted Ehrhart quasi-polynomial has period " << period << endl;    
+        }
+    }
+    
+   out  << endl << endl;
+    
+   if (HS.isHilbertQuasiPolynomialComputed()){    
+    long deg=HS.getHilbertQuasiPolynomial()[0].size()-1;
+        out << "Degree of (quasi)polynomial: " << deg << endl;
+    
+    long virtDeg=Result->getRank()+Result->getIntData().getDegreeOfPolynomial()-1;
+
+    out << endl << "Expected degree: " <<  virtDeg << endl;
+    }
+        
+    if(Result->isComputed(ConeProperty::VirtualMultiplicity)){
+        out << endl << "Virtual multiplicity: ";
+        out << Result->getIntData().getVirtualMultiplicity() << endl;
+        if(Result->getIntData().getVirtualMultiplicity().get_den()!=1)
+                out << "Virtual multiplicity (float): "<< std::setprecision(12) << mpq_to_nmz_float(Result->getIntData().getVirtualMultiplicity()) << endl;
+        out << endl;
+    }
+}
 
 //---------------------------------------------------------------------------
 
@@ -609,14 +780,14 @@ void Output<Integer>::write_files() const {
             out << nr_orig_gens <<" original generators of the toric ring"<<endl;
         }
         if (Result->isComputed(ConeProperty::ModuleGenerators)) {
-            out << Result->getNrModuleGenerators() <<" module generators" << endl;
+            out << Result->getNrModuleGenerators() << module_generators_name << endl;
         }
         if (Result->isComputed(ConeProperty::HilbertBasis)) {
             out << Result->getNrHilbertBasis() <<" Hilbert basis elements"
                 << of_monoid << endl;
         }
         if (homogeneous && Result->isComputed(ConeProperty::Deg1Elements)) {
-            out << Result->getNrDeg1Elements() <<" Hilbert basis elements of degree 1"<<endl;
+            out << Result->getNrDeg1Elements() << module_generators_name << endl;
         }
         if (Result->isComputed(ConeProperty::IsReesPrimary)
             && Result->isComputed(ConeProperty::HilbertBasis)) {
@@ -658,8 +829,12 @@ void Output<Integer>::write_files() const {
             if (Result->isComputed(ConeProperty::AffineDim))
                 out << "affine dimension of the polyhedron = "
                     << Result->getAffineDim() << is_maximal(Result->getAffineDim(),dim-1) << endl;
-            if (Result->isComputed(ConeProperty::RecessionRank))
-                out << "rank of recession monoid = "  << Result->getRecessionRank() << endl;
+            if (Result->isComputed(ConeProperty::RecessionRank)){
+                out << "rank of recession monoid = "  << Result->getRecessionRank();
+                if(Result->getRecessionRank()==0)
+                    out << " (polyhedron is polytope)";                
+                out << endl;
+            }
         }
         
         if(Result->isComputed(ConeProperty::OriginalMonoidGenerators)){
@@ -743,8 +918,17 @@ void Output<Integer>::write_files() const {
         }
         if ( Result->isComputed(ConeProperty::Multiplicity) ) {
             out << "multiplicity = "<< Result->getMultiplicity() << endl;
+            if(Result->getMultiplicity().get_den()!=1)
+                out << "multiplicity (float) = "<< std::setprecision(12) << mpq_to_nmz_float(Result->getMultiplicity()) << endl;
         }
-        if ( Result->isComputed(ConeProperty::ModuleRank) || Result->isComputed(ConeProperty::Multiplicity)) {
+        if ( Result->isComputed(ConeProperty::Volume) && Result->isComputed(ConeProperty::Sublattice)) {
+            out << "volume (normalized) = "<< std::setprecision(12) << Result->getVolume() << endl;
+            if(Result->getVolume().get_den()!=1)
+                out << "volume (normalized, float) = "<< mpq_to_nmz_float(Result->getVolume()) << endl;
+            out << "volume (Euclidean) = "<< std::setprecision(12) << Result->getEuclideanVolume() << endl;
+        }
+        if ( Result->isComputed(ConeProperty::ModuleRank) || Result->isComputed(ConeProperty::Multiplicity) 
+                    || Result->isComputed(ConeProperty::Volume)) {
             out << endl;
         }
         
@@ -777,6 +961,13 @@ void Output<Integer>::write_files() const {
             if(v_is_symmetric(HS_Num)){
                 out << "The numerator of the Hilbert Series is symmetric." << endl << endl;
             }
+            if(HS.get_expansion_degree()>-1){
+                vector<mpz_class> expansion=HS.getExpansion();
+                out << "Expansion of Hilbert series" << endl;
+                for(size_t i=0;i<expansion.size();++i)
+                    out << i+HS.getShift()  << ": " << expansion[i] << endl;
+                out << endl;
+            }
             long period = HS.getPeriod();
             if (period == 1 && (HS_Denom.size() == 0
                                 || HS_Denom.begin()->first== (long) HS_Denom.size())) {
@@ -796,6 +987,10 @@ void Output<Integer>::write_files() const {
                 HS.computeHilbertQuasiPolynomial();
                 if (HS.isHilbertQuasiPolynomialComputed()) {
                     out<<"Hilbert quasi-polynomial of period " << period << ":" << endl;
+                    if(HS.get_nr_coeff_quasipol()>=0){
+                        out << "only " << HS.get_nr_coeff_quasipol() << " highest coefficients computed" << endl;
+                        out << "their common period is " << HS.getHilbertQuasiPolynomial().size() << "" << endl;                        
+                    }
                     Matrix<mpz_class> HQP(HS.getHilbertQuasiPolynomial());
                     HQP.pretty_print(out,true);
                     out<<"with common denominator = "<<HS.getHilbertQuasiPolynomialDenom();
@@ -806,6 +1001,25 @@ void Output<Integer>::write_files() const {
             }
 
         }
+ 
+        if ( Result->isComputed(ConeProperty::WeightedEhrhartSeries) )
+            writeWeightedEhrhartSeries(out);
+            
+        
+        if ( Result->isComputed(ConeProperty::VirtualMultiplicity)
+                   && !Result->isComputed(ConeProperty::WeightedEhrhartQuasiPolynomial)) {
+            out << "virtual multiplicity = "<< Result->getVirtualMultiplicity() << endl;
+            if(Result->getVirtualMultiplicity().get_den()!=1)
+                out << "virtual multiplicity (float) = "<< std::setprecision(12) << mpq_to_nmz_float(Result->getVirtualMultiplicity()) << endl;
+            out << endl;
+        }
+        
+        if ( Result->isComputed(ConeProperty::Integral)) {
+            out << "integral  = "<< Result->getIntegral() << endl;
+            if(Result->getIntegral().get_den()!=1)
+                out << "integral (float) = "<< std::setprecision(12) << mpq_to_nmz_float(Result->getIntegral()) << endl;
+            out << endl;
+        }
 
         if (Result->isComputed(ConeProperty::IsReesPrimary)) {
             if (Result->isReesPrimary()) {
@@ -831,6 +1045,17 @@ void Output<Integer>::write_files() const {
                 out << endl;
             }                   
         }
+        
+        if(Result->isComputed(ConeProperty::IsGorenstein)) {
+            if(Result->isGorenstein()){
+                out << "Monoid is Gorenstein " << endl;
+                out << "Generator of interior:" << endl;
+                out << Result->getGeneratorOfInterior();              
+            }
+            else
+                out << "Monoid is not Gorenstein " << endl;
+            out << endl;
+        }
 
         out << "***********************************************************************"
             << endl << endl;
@@ -842,7 +1067,7 @@ void Output<Integer>::write_files() const {
             out << endl;
         }
         if (Result->isComputed(ConeProperty::ModuleGenerators)) {
-            out << Result->getNrModuleGenerators() <<" module generators:" << endl;
+            out << Result->getNrModuleGenerators() << module_generators_name <<  ":" << endl;
             Result->getModuleGeneratorsMatrix().pretty_print(out);
             out << endl;
         }
@@ -851,7 +1076,7 @@ void Output<Integer>::write_files() const {
             const Matrix<Integer>& Hom = Result->getDeg1ElementsMatrix();
             write_matrix_ht1(Hom);
             nr=Hom.nr_of_rows();
-            out<<nr<<" Hilbert basis elements of degree 1:"<<endl;
+            out<<nr << module_generators_name << ":"<<endl;
             Hom.pretty_print(out);
             out << endl;
         }
@@ -914,24 +1139,31 @@ void Output<Integer>::write_files() const {
                 out << endl;
             }
         }
-        if (Result->isComputed(ConeProperty::VerticesOfPolyhedron)) {
+        if (Result->isComputed(ConeProperty::VerticesOfPolyhedron)  && !no_ext_rays_output) {
             out << Result->getNrVerticesOfPolyhedron() <<" vertices of polyhedron:" << endl;
-            Result->getVerticesOfPolyhedronMatrix().pretty_print(out);
+            if(Result->isComputed(ConeProperty::VerticesFloat))
+                Result->getVerticesFloatMatrix().pretty_print(out); // write_float(out,Result->getVerticesFloatMatrix(),Result->getNrVerticesFloat(),dim);
+            else
+                Result->getVerticesOfPolyhedronMatrix().pretty_print(out);
             out << endl;
         }
-        if (Result->isComputed(ConeProperty::ExtremeRays)) {
+        if (Result->isComputed(ConeProperty::ExtremeRays) && !no_ext_rays_output) {
             out << Result->getNrExtremeRays() << " extreme rays" << of_cone << ":" << endl;
-            Result->getExtremeRaysMatrix().pretty_print(out);
+            if(homogeneous && Result->isComputed(ConeProperty::VerticesFloat))
+                Result->getVerticesFloatMatrix().pretty_print(out); // write_float(out,Result->getVerticesFloatMatrix(),Result->getNrVerticesFloat(),dim); 
+            else
+                Result->getExtremeRaysMatrix().pretty_print(out);
             out << endl;
-            if (ext) {
-                // for the .gen file we append the vertices of polyhedron if there are any
-                if (Result->isComputed(ConeProperty::VerticesOfPolyhedron)) {
-                    Matrix<Integer> Extreme_Rays(Result->getExtremeRaysMatrix());
-                    Extreme_Rays.append(Result->getVerticesOfPolyhedronMatrix());
-                    write_matrix_ext(Extreme_Rays);
-                } else {
-                    write_matrix_ext(Result->getExtremeRaysMatrix());
-                }
+        }
+        
+        if (Result->isComputed(ConeProperty::ExtremeRays) && ext){
+            // for the .gen file we append the vertices of polyhedron if there are any
+            if (Result->isComputed(ConeProperty::VerticesOfPolyhedron)) {
+                Matrix<Integer> Extreme_Rays(Result->getExtremeRaysMatrix());
+                Extreme_Rays.append(Result->getVerticesOfPolyhedronMatrix());
+                write_matrix_ext(Extreme_Rays);
+            } else {
+                write_matrix_ext(Result->getExtremeRaysMatrix());
             }
         }
         
@@ -953,17 +1185,22 @@ void Output<Integer>::write_files() const {
 
         //write constrains (support hyperplanes, congruences, equations)
 
-        if (Result->isComputed(ConeProperty::SupportHyperplanes)) {
+        if (Result->isComputed(ConeProperty::SupportHyperplanes) && !no_supp_hyps_output) {
             const Matrix<Integer>& Support_Hyperplanes = Result->getSupportHyperplanesMatrix();
             out << Support_Hyperplanes.nr_of_rows() <<" support hyperplanes" 
                 << of_polyhedron << ":" << endl;
-            Support_Hyperplanes.pretty_print(out);
+            if(Result->isComputed(ConeProperty::SuppHypsFloat))
+                Result->getSuppHypsFloatMatrix().pretty_print(out);
+            else
+                Support_Hyperplanes.pretty_print(out);
             out << endl;
         }
         if (Result->isComputed(ConeProperty::Sublattice)) {
             const Sublattice_Representation<Integer>& BasisChange = Result->getSublattice();
             //equations
-            const Matrix<Integer>& Equations = BasisChange.getEquationsMatrix();
+            const Matrix<Integer>& EQ = BasisChange.getEquationsMatrix();
+            Matrix<Integer> Equations=EQ;
+            Equations.row_echelon_reduce();
             size_t nr_of_equ = Equations.nr_of_rows();
             if (nr_of_equ > 0) {
                 out << nr_of_equ <<" equations:" <<endl;
@@ -981,7 +1218,9 @@ void Output<Integer>::write_files() const {
             }
             
             //lattice
-            const Matrix<Integer>& LatticeBasis = BasisChange.getEmbeddingMatrix();
+            const Matrix<Integer>& LB = BasisChange.getEmbeddingMatrix();
+            Matrix<Integer> LatticeBasis=LB;
+            LatticeBasis.row_echelon_reduce();
             size_t nr_of_latt = LatticeBasis.nr_of_rows();
             if (nr_of_latt < dim ||  BasisChange.getExternalIndex()!=1) {
                 out << nr_of_latt <<" basis elements of lattice:" <<endl;
